package org.richfaces.component;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import javax.faces.component.UIComponent;
import javax.faces.component.UIComponentBase;
import javax.faces.component.UIForm;
import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;

public abstract class UIFocus extends UIComponentBase {

    public static final String COMPONENT_TYPE = "org.richfaces.Focus";
    public static final String COMPONENT_FAMILY = "org.richfaces.Focus";

    public Integer getDefaultPriority() {
        UIComponent parentForm = getParent();
        while (parentForm != null && !(parentForm instanceof UIForm)) {
            parentForm = parentForm.getParent();
        }
        if (parentForm != null) {
            return getUIInputChildrenCount((UIForm) parentForm, getForComponentId());
        } else {
            return Integer.MAX_VALUE;
        }
    }

    public String getForComponentId() {
        String aFor = getFor();

        if (aFor != null && !"".equals(aFor)) {
            return aFor;
        } else {
            if (!(getParent() instanceof UIInput)) {
                Set<String> allowedClientIds = new HashSet<String>();
                Iterator<String> clientIdsWithMessages = getFacesContext().getClientIdsWithMessages();
                while (clientIdsWithMessages.hasNext()) {
                    allowedClientIds.add(clientIdsWithMessages.next());
                }
                UIComponent component = getFirstInput(getParentForm(this), allowedClientIds);
                return component == null ? null : component.getId();
            } else {
                return getParent().getId();
            }
        }
    }

    private int getUIInputChildrenCount(UIComponent component, String breakOnId) {
        int inputComponentsCount = 0;
        for (UIComponent child : component.getChildren()) {
            if (child.getId().equals(breakOnId)) {
                break;
            }
            if (child instanceof UIInput) {
                inputComponentsCount++;
            } else {
                int uIInputChildrenCount = getUIInputChildrenCount(child, breakOnId);
                inputComponentsCount += uIInputChildrenCount;
            }
        }
        return inputComponentsCount;
    }

    public abstract String getFor();

    public abstract void setFor(String value);

    public abstract String getSuffix();

    public abstract void setSuffix(String value);

    public abstract Integer getPriority();

    public abstract void setPriority(Integer value);

    private UIComponent getFirstInput(UIComponent parent, Set<String> allowedClientIds) {
        UIComponent input = null;
        FacesContext facesContext = getFacesContext();
        for (UIComponent child : parent.getChildren()) {
            if (child instanceof UIInput && (allowedClientIds.size() == 0 || allowedClientIds.contains(child.getClientId(facesContext)))) {
                return child;
            } else {
                input = getFirstInput(child, allowedClientIds);
                if (input != null) {
                    break;
                }
            }
        }
        return input;
    }

    private UIForm getParentForm(UIComponent component) {
        UIComponent parent = component.getParent();
        if (parent == null) {
            return null;
        }
        if (parent instanceof UIForm) {
            return (UIForm) parent;
        } else {
            return getParentForm(parent);
        }
    }
}
